<?php

namespace Picea\Asset\Action;

use Picea\Asset\Config;
use Picea\Asset\FileFetcher;

class Symlink implements InstallActionInterface
{
    public array $packageLinks = [];

    public function __construct(
        protected FileFetcher $fileFetcher,
        protected Config $config,
    ) { }

    public function run(bool $dryRun = false) : mixed
    {
        if (empty($this->config->destination)) {
            throw new \InvalidArgumentException("A destination must be provided into Config object.");
        }

        foreach($this->fileFetcher->getFileList() as $path => $file) {
            $this->mapPackagesLinks($path, $file);
        }

        if ( ! $dryRun ) {
            foreach($this->packageLinks as $link => $source) {
                $destination = $this->config->destination . DIRECTORY_SEPARATOR . $link;

                # Clear previously linked path
                if (is_link($destination)) {
                    unlink($destination);
                }

                if ( ! @symlink($source, $destination) ) {
                    throw new \RuntimeException(sprintf("Could not create symlink for %s -> %s", $source, $destination));
                }
            }
        }

        return null;
    }

    protected function mapPackagesLinks(string $path, array $splFiles) : void
    {
        $links = [];
        $len = strlen($path);

        foreach($splFiles as $file) {
            $name = $file->getPathName();
            $relativePath = substr($name, $len);

            $link = $this->extractLink($relativePath);

            $links[$link] ??= $path . $link;
        }

        $this->packageLinks = $this->packageLinks + $links;
    }

    protected function extractLink(string $name) {
        $pos = strpos($name, DIRECTORY_SEPARATOR);

        return $pos !== false ? substr($name, 0, $pos) : $name;
    }
}